"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getVarNameAndFunc = exports.COLOR_FN_RE = exports.fadeout = exports.fadein = exports.shade = exports.tint = exports.fade = exports.darken = exports.lighten = exports.staticColorList = exports.COLOR_FUNC_NAME_LIST = exports.__ANT_DESIGN_FUNCTION_TOKEN__ = void 0;
const tinycolor2_1 = __importDefault(require("tinycolor2"));
exports.__ANT_DESIGN_FUNCTION_TOKEN__ = '_T_';
exports.COLOR_FUNC_NAME_LIST = [
    'color',
    'darken',
    'tint',
    'fade',
    'lighten',
    'shade',
    'fadeout',
    'fadein',
    'colorPalette',
];
exports.staticColorList = exports.COLOR_FUNC_NAME_LIST.filter((item) => ~['color', 'colorPalette'].includes(item));
function clamp(val) {
    return Math.min(1, Math.max(0, val));
}
const lighten = (color, amount) => (0, tinycolor2_1.default)(color).lighten(amount).toString();
exports.lighten = lighten;
const darken = (color, amount) => (0, tinycolor2_1.default)(color).darken(amount).toString();
exports.darken = darken;
// https://github.com/less/less.js/blob/b37922cfb2932f00d1e8b340f4799ff24a2af0f2/packages/less/src/less/functions/color.js#L318
const fade = (color, amount = 10) => {
    const hsl = (0, tinycolor2_1.default)(color).toHsl();
    hsl.a = amount / 100;
    hsl.a = clamp(hsl.a);
    return (0, tinycolor2_1.default)(hsl).toString();
};
exports.fade = fade;
// https://github.com/less/less.js/blob/b37922cfb2932f00d1e8b340f4799ff24a2af0f2/packages/less/src/less/functions/color.js#L444
const tint = (color, amount = 10) => {
    return tinycolor2_1.default.mix('#ffffff', color, amount).toString();
};
exports.tint = tint;
const shade = (color, amount = 10) => {
    return tinycolor2_1.default.mix('#000000', color, amount).toString();
};
exports.shade = shade;
const fadein = (color, amount, method) => {
    const hsl = (0, tinycolor2_1.default)(color).toHsl();
    if (typeof method !== 'undefined' && method.value === 'relative') {
        hsl.a += (hsl.a * amount.value) / 100;
    }
    else {
        hsl.a += amount.value / 100;
    }
    hsl.a = clamp(hsl.a);
    return (0, tinycolor2_1.default)(hsl).toString();
};
exports.fadein = fadein;
const fadeout = (color, amount, method) => {
    const hsl = (0, tinycolor2_1.default)(color).toHsl();
    if (typeof method !== 'undefined' && method.value === 'relative') {
        hsl.a -= (hsl.a * amount.value) / 100;
    }
    else {
        hsl.a -= amount.value / 100;
    }
    hsl.a = clamp(hsl.a);
    return (0, tinycolor2_1.default)(hsl).toString();
};
exports.fadeout = fadeout;
// card-skeleton-bg--fade--40
// ! 性能问题
exports.COLOR_FN_RE = new RegExp(`(.*)+--(${exports.COLOR_FUNC_NAME_LIST.join('|')})--(\\d+)`);
function getVarNameAndFunc(key) {
    // const match = key.match(COLOR_FN_RE);
    const [varName, funcName, varNumber] = key.split(new RegExp(`--(${exports.COLOR_FUNC_NAME_LIST.join('|')})--`));
    return { varName, funcName, varNumber };
    // if (match) {
    //   const [, varName, funcName, varNumber] = match;
    //   return { varName, funcName, varNumber };
    // }
}
exports.getVarNameAndFunc = getVarNameAndFunc;
// export const colorPalette = (function () {
//   let hueStep = 2;
//   let saturationStep = 16;
//   let saturationStep2 = 5;
//   let brightnessStep1 = 5;
//   let brightnessStep2 = 15;
//   let lightColorCount = 5;
//   let darkColorCount = 4;
//   let getHue = function (hsv, i, isLight) {
//     let hue;
//     if (hsv.h >= 60 && hsv.h <= 240) {
//       hue = isLight ? hsv.h - hueStep * i : hsv.h + hueStep * i;
//     } else {
//       hue = isLight ? hsv.h + hueStep * i : hsv.h - hueStep * i;
//     }
//     if (hue < 0) {
//       hue += 360;
//     } else if (hue >= 360) {
//       hue -= 360;
//     }
//     return Math.round(hue);
//   };
//   let getSaturation = function (hsv, i, isLight) {
//     let saturation;
//     if (isLight) {
//       saturation = Math.round(hsv.s * 100) - saturationStep * i;
//     } else if (i === darkColorCount) {
//       saturation = Math.round(hsv.s * 100) + saturationStep;
//     } else {
//       saturation = Math.round(hsv.s * 100) + saturationStep2 * i;
//     }
//     if (saturation > 100) {
//       saturation = 100;
//     }
//     if (isLight && i === lightColorCount && saturation > 10) {
//       saturation = 10;
//     }
//     if (saturation < 6) {
//       saturation = 6;
//     }
//     return Math.round(saturation);
//   };
//   let getValue = function (hsv, i, isLight) {
//     if (isLight) {
//       return Math.round(hsv.v * 100) + brightnessStep1 * i;
//     }
//     return Math.round(hsv.v * 100) - brightnessStep2 * i;
//   };
//   return (color, index) => {
//     var isLight = index <= 6;
//     var hsv = tinycolor(color).toHsv();
//     var i = isLight ? lightColorCount + 1 - index : index - lightColorCount - 1;
//     return tinycolor({
//       h: getHue(hsv, i, isLight),
//       s: getSaturation(hsv, i, isLight),
//       v: getValue(hsv, i, isLight),
//     }).toHexString();
//   };
// })();
