"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runLessToCssVariables = void 0;
const generate_1 = require("./generate");
const rules_1 = require("./rules");
const reduction_1 = require("./reduction");
const resolve_1 = require("./utils/resolve");
const chalk_1 = __importDefault(require("chalk"));
const path_1 = __importDefault(require("path"));
const variables_1 = require("./generate/variables");
async function runLessToCssVariables({ cwd = process.cwd(), revert = false, force = false, debug = false, verbose = false, libraryList = [], series = 'ant-design', } = {}) {
    const libEntry = (0, resolve_1.resolveLibraryList)(libraryList);
    await Promise.all(libEntry.map(async (libraryDir, index) => {
        const libraryItem = libraryList[index];
        const rules = await (0, rules_1.buildRules)(libraryItem, series, force);
        const opts = {
            cwd,
            force,
            revert,
            verbose,
            debug,
            libraryDir,
            libraryItem,
            joinFullPath: (...paths) => path_1.default.join(libraryDir, ...paths),
        };
        try {
            if (revert) {
                return (0, reduction_1.runReductions)(rules, opts).then(() => {
                    console.info(` Restore the ${chalk_1.default.cyan(libraryItem.name)} style variable file successfully`);
                });
            }
            // 执行修改、覆盖
            return (0, generate_1.generateCoverages)(rules, opts).then(() => {
                console.info(`Successfully overwrite and modify the ${chalk_1.default.cyan(libraryItem.name)} style variable file`);
            });
        }
        catch (error) {
            console.error(`[preprocess-less-to-css-variable]: ${error.toString()}`);
        }
    }));
    // 处理变量，写入缓存
    await (0, variables_1.generateVariables)();
}
exports.runLessToCssVariables = runLessToCssVariables;
